#コマ
class Panel < ActiveRecord::Base
  belongs_to :author
  belongs_to :resource_picture
  has_many :stories
  has_many :panel_pictures, :dependent => :destroy
  has_many :speech_balloons, :dependent => :destroy
  has_many :ground_pictures, :dependent => :destroy
  has_many :ground_colors, :dependent => :destroy
  has_many :panel_colors, :dependent => :destroy
  accepts_nested_attributes_for :panel_pictures, :allow_destroy => true
  accepts_nested_attributes_for :speech_balloons, :allow_destroy => true
  accepts_nested_attributes_for :ground_pictures, :allow_destroy => true
  accepts_nested_attributes_for :ground_colors, :allow_destroy => true
  accepts_nested_attributes_for :panel_colors, :allow_destroy => true

  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :border, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  validates :x, :numericality => {:allow_blank => true}
  validates :y, :numericality => {:allow_blank => true}
  validates :z, :numericality => {:allow_blank => true, :greater_than => 0}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :publish, :presence => true, :numericality => true
  
  def supply_default
    self.border = 2
    self.publish = 0
  end
  
  def overwrite au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = Panel.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.own?(roles)
    self.publish?
  end
  
  def usable? au
    visible? au
  end
  
  def publish?
    self.publish > 0
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(self.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'panels.updated_at desc'})
    Panel.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {
      :panel_pictures => {
        :picture => {:artist => {}, :license => {}}
      }, 
      :speech_balloons => {:balloons => {}, :speeches => {}},
      :ground_pictures => {
        :picture => {:artist => {}, :license => {}}
      }, 
      :ground_colors => {
        :color => {}
      }, 
      :panel_colors => {
      }, 
      :author => {}
    }}
  end
  
  def self.mylist au, page = 1, page_size = Author.default_panel_page_size
    opt = {}
    opt.merge!(self.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['panels.author_id = ?', au.id], :order => 'panels.updated_at desc'})
    Panel.find(:all, opt)
  end
  
  def self.show rid, roles
    opt = {}
    opt.merge!(Panel.show_opt)
    res = Panel.find(rid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit rid, au
    opt = {}
    opt.merge!(Panel.show_opt)
    res = Panel.find(rid, opt)
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {
      :panel_pictures => {
        :picture => {:artist => {}, :license => {}}
      }, 
      :speech_balloons => {:balloons => {}, :speeches => {}},
      :ground_pictures => {
        :picture => {:artist => {}, :license => {}}
      }, 
      :ground_colors => {
        :color => {}
      }, 
      :panel_colors => {
      }, 
      :author => {}
    }}
  end
  
  def parts_element
    ((self.panel_pictures || []) + (self.speech_balloons || [])).compact
  end
  
  def parts
    res = []
    self.parts_element.each do |e|
      res[e.t] = e
    end
    res
  end
  
  def grounds
    ((self.ground_colors || []) + (self.ground_pictures || []) + (self.panel_colors || [])).compact
  end
  
  def panel_elements
    parts + grounds
  end
  
  @@elm_json_opt = {
    'PanelPicture' => {
      :picture => {:artist => {}, :license => {}}
    }, 
    'SpeechBalloon' => {:balloons => {}, :speeches => {}},
    'GroundPicture' => {
      :picture => {:artist => {}, :license => {}}
    }, 
    'GroundColor' => {
      :color => {}
    }, 
    'PanelColor' => {
    } 
  }
  
  def self.elm_json_opt e
    @@elm_json_opt[e.class.to_s]
  end
  
  def elements
    self.panel_elements.map {|e|
      #(-_-;)<... kore wa hidoi
      JSON.parse e.to_json({:include => Panel.elm_json_opt(e)})
    }
  end
  
  def panel_elements_as_json
    self.to_json({:include => {:author => {}}, :methods => :elements})
  end
  
  def self.list_as_json_text ary
    '[' + ary.map {|i| i.panel_elements_as_json }.join(',') + ']'
  end
  
  def licensed_pictures
    r = {}
    ((self.panel_pictures || []) + (self.ground_pictures || [])).compact.each do |elm|
      r[elm.picture_id] = elm.picture unless r[elm.picture_id]
    end
    r
  end
  
  def self.visible_count
    Panel.count
  end
  
  def self.collect_element_value elements, name
    elements.map {|e|
      e.map {|o|
        o[name]
      }
    }.flatten
  end
  
  def self.validate_t ary
    i = 0
    ary.compact.sort.each do |t|
      break false unless t == i
      i += 1
    end
    ary.compact.size == i
  end
  
  def self.validate_element_t elements, name
    Panel.validate_t(Panel.collect_element_value(elements, name))
  end
  
  def self.validate_elements_t c
    c.map {|conf|
      Panel.validate_element_t(conf[:elements], conf[:name]) ? nil : false
    }.compact.empty?
  end
  
  def validate_t_list
    [
      {:elements => [self.panel_pictures, self.speech_balloons], :name => :t}
    ]
  end
  def validate_child
#    r1 = Panel.validate_elements_id validate_id_list
    Panel.validate_elements_t validate_t_list
  end
  
  def store attr, au
    if attr == false
      self.errors.add :base, I18n.t('errors.invalid_json')
      return false
    end
    self.attributes = attr
    self.overwrite au
    res = false
    Panel.transaction do
      unless validate_child
        self.errors.add :base, I18n.t('errors.invalid_t')
        raise ActiveRecord::Rollback
      end
      res = self.save
    end
    res
  end
  
  def destroy_with_elements
    res = false
    Panel.transaction do
      self.panel_elements.each do |element|
        raise ActiveRecord::Rollback unless element.destroy
      end
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
=begin
  def self.validate_id ary, pid
    ary.map {|v|
      if pid
        (v == pid or v == nil) ? nil : false
      else
        v ? false : nil
      end
    }.compact.empty?
  end
  
  def self.validate_element_id elements, name, pid
    Panel.validate_id(Panel.collect_element_value(elements, name), pid)
  end
  
  def self.validate_elements_id c
    c.map {|conf|
      Panel.validate_element_id(conf[:elements], conf[:name], conf[:parent_id]) ? nil : false
    }.compact.empty?
  end
  
  def validate_id_list
    r = self.speech_balloons.map {|sb|
      {:elements => [sb.speeches, sb.balloons], :name => :speech_balloon_id, :parent_id => sb.id}
    }
    r.unshift({:elements => [self.panel_pictures, self.speech_balloons], :name => :panel_id, :parent_id => self.id})
    r
  end
=end
  
end
